% File: pde_1_main_3D.m
% From Appendix 4 - 'Implementation of Time-Varying Boundary Conditions',
%                   of the book:
% William E Schiesser and Graham W Griffiths (2009).
% A Compendium of Partial Differential Equation Models
% - Method of Lines Analysis with Matlab,
% Cambridge University Press (ISBN-13: 9780521519861).
%
% Clear previous files
  clear all
  clc
%
% Parameters shared with the ODE routine
  global a D n xl xu dx ncall ndss
%
% Parameters
  a=1.0;
  D=0.3;
  xl=0.0;
  xu=1.0;
%
% Cases with the time varying Dirichlet boundary condition
% (TVDBC) in or outside the spatial grid
  mf=5;
%
% mf = 1,2,3; TVDBC not part of the spatial grid
%
%   mf = 1 - explicit programming of the FDs with boundary value
%            calculated for the ODE at x = dx in pde_1.m
%
%   mf = 2 - ux routines dss002 to dss010 with boundary value
%            calculated for MOL ODEs in pde_2.m
%
%   mf = 3 - uxx routines dss042 to dss050 with boundary value
%            calculated for MOL ODEs in pde_3.m
%
%   For mf = 1,2,3, the boundary value must be calculated after
%   the return from the ODE integrator if it is to be displayed
%   and/or plotted.
    if(mf<=3)
      n=20;
      dx=(xu-xl)/n;
    end  
%
% mf = 4,5,6,7; TVDBC part of the spatial grid
%
%   mf = 4 - explicit programming of the FDs with boundary value
%            calculated at boundary in pde_4.m; this boundary
%            value is not returned by the ODE integrator and
%            it must therefore be restored after the return 
%            from the integrator if it is to be displayed and/or 
%            plotted
%
%   mf = 5 - explicit programming of the FDs with boundary value
%            calculated and derivative of boundary value also 
%            calculated (for ODE at boundary) in pde_5.m; the
%            ODE integrator returns the correct boundary value
%
%   mf = 6 - ux routines dss002 to dss010 with boundary value
%            calculated for MOL ODEs in pde_6.m; the ODE integrator
%            returns the correct boundary value
%
%   mf = 7 - uxx routines dss042 to dss050 with boundary value
%            calculated for MOL ODEs in pde_7.m; the ODE integrator
%            returns the correct boundary value
    if(mf>3)
      n=21;
      dx=(xu-xl)/(n-1);
    end 
%
% Spatial grid  
  x=[xl:dx:xu]';  
%  
% Initial condition
  u0=zeros(n,1);
%
% Independent variable for ODE integration
  t0=0.0;
  tf=2.5;
%   tout=[0.0:0.25:tf]'; 
%   nout=11;
nout = 51;
tout=linspace(t0,tf,nout);
  ncall=0;
%
% ODE itegration
  reltol=1.0e-04; abstol=1.0e-04;
  options=odeset('RelTol',reltol,'AbsTol',abstol);
  if(mf==1) % explicit FDs 
    [t,u]=ode15s(@pde_1,tout,u0,options); end 
  if(mf==2) ndss=4; % ndss = 2, 4, 6, 8 or 10 required
    [t,u]=ode15s(@pde_2,tout,u0,options); end
  if(mf==3) ndss=44; % ndss = 42, 44, 46, 48 or 50 required 
    [t,u]=ode15s(@pde_3,tout,u0,options); end
  if(mf==4) % explicit FDs 
    [t,u]=ode15s(@pde_4,tout,u0,options); end 
  if(mf==5) % explicit FDs 
    [t,u]=ode15s(@pde_5,tout,u0,options); end 
  if(mf==6) ndss=4; % ndss = 2, 4, 6, 8 or 10 required
    [t,u]=ode15s(@pde_6,tout,u0,options); end 
  if(mf==7) ndss=44; % ndss = 42, 44, 46, 48 or 50 required 
    [t,u]=ode15s(@pde_7,tout,u0,options); end
%
% Display selected output
  if(mf==1)
    n2=n/2; 
    fprintf('\n                    mf = %2d   n = %3d',mf,n);
  elseif(mf==2|mf==3)
    n2=n/2;      
    fprintf('\n             mf = %2d   n = %3d   ndss = %2d',mf,n,ndss);
  elseif(mf==4|mf==5)
    n2=(n-1)/2+1;
    fprintf('\n                    mf = %2d   n = %3d',mf,n); 
  elseif(mf==6|mf==7) 
    n2=(n-1)/2+1;       
    fprintf('\n             mf = %2d   n = %3d   ndss = %2d',mf,n,ndss);
  end  
  fprintf('\n         abstol = %8.1e   reltol = %8.1e',abstol,reltol);
  if(mf<=3)
    fprintf('\n\n     t   u(x=dx,t)  u(x=0.5,t)    u(x=1,t)\n');
  else
    fprintf('\n\n     t    u(x=0,t)  u(x=0.5,t)    u(x=1,t)\n'); 
  end
  for it=1:nout
    if(mf==4)u(it,1)=1-cos(a*pi*t(it));end
    fprintf('%6.2f%12.6f%12.6f%12.6f\n',t(it),u(it,1),u(it,n2),u(it,n));
  end
  fprintf('\n ncall = %4d\n',ncall);
%
% Plot numerical solution as u(x,t) vs x with t as parameter
  if(mf<=3)
    uplot(:,2:n+1)=u(:,1:n);
%
%   Include boundary value in plot
    for it=1:nout
      uplot(it,1)=1-cos(a*pi*t(it));
    end
  elseif(mf>3)
    uplot=u;
%   
%   Restore boundary value for plot (mf = 4)
    if(mf==4)
      for it=1:nout
        uplot(it,1)=u(it,1);
      end
    end  
  end  
  figure(1);
  plot(x,uplot,'-k'); axis tight
  title('u(x,t) vs x, t = 0,0.25,...,2.5');
  xlabel('x');
  ylabel('u(x,t)')
  figure(2);
  surf(x,tout,uplot);
  shading interp; axis tight
  colormap jet
  title('u(x,t) vs x, t = 0,0.25,...,2.5');
  xlabel('x'); 
  ylabel('t'); 
  zlabel('u(x,t)');
  rotate3d on
% print -deps pde.eps; print -dps pde.ps
