% File: sphericalDiffusion_fPlot.m (rev. 1 - October 2012)
% From Appendix 6 - 'Animating Simulation Results', of the book:
% William E Schiesser and Graham W Griffiths (2009).
% A Compendium of Partial Differential Equation Models
% - Method of Lines Analysis with Matlab,
% Cambridge University Press (ISBN-13: 9780521519861).
%
%==========================================================
% Creates x-y and x-z plane polar plots of inhomogeneous 2D
% Gaussian source term 'f(r,theta,phi)' from pde_2_main.m 
% the from Matlab workspace data generated by the code of 
% Chapter 14. Note: f is symmetrical about phi.
%==========================================================
%
% Clear previous files
  clc; clear all
%
% Model parameters
  D=0.1;
  r0=3; th0=2*pi;
  std_0=1.0; std_pi2=2.0;
  tau=1.0;
  NN=101;
%
% Radial grid
  nr=NN; r=linspace(0,r0,nr);
%
% Angular grid
  nth=NN; th=linspace(0,th0,nth);
%
% Create polar mesh
  [R, Theta]=meshgrid(r, th);
  [X1,Y]=pol2cart(Theta,R); % Convert to Cartesian coordinates
  [X2,Z]=pol2cart(Theta,R); % Convert to Cartesian coordinates
%
% Calculate Inhomogeneous function data
F1=zeros(nr,nth);
F2=zeros(nr,nth);
  for i=1:nr
    for j=1:nth
      F1(i,j)=exp(-(r(i)/std_0 )^2); % Symmetrical about phi
      F2(i,j)=exp(-(r(i)*sin(th(j))/std_pi2)^2 ...
                  -(r(i)*cos(th(j))/std_0 )^2);
    end
  end
%
% Convert function data to be suitable for use with
% the meshgrid using cubic interpolation
  Fxy=interp2(th,r,F1,Theta,R,'linear');
  Fxz=interp2(th,r,F2,Theta,R,'linear');
%
% Plot r-phi (x-y) plane of inhomogeneous source,
% f(r,theta,phi) over four quadrants (uses Cartesian
% coordinate data)
  figure()
  subplot(1,2,1)
  surf(X1, Y, Fxy); shading interp;
  axis square; axis tight
  caxis([0,1]); colormap jet
  xlabel('X'); ylabel('Y'); zlabel('f(r,\theta)');
  title({'f(r,\theta,\phi) - (r- \phi) Plane'});
  view([0,90]);
%
% Plot r-theta (x-z) plane of inhomogeneous source,
% f(r,theta,phi) over four quadrants (uses Cartesian
% coordinate data)
  subplot(1,2,2)
  surf(X2, Z, Fxz); shading interp;
  axis square; axis tight
  caxis([0,1]); colormap jet
  xlabel('X'); ylabel('Z'); zlabel('f(r,\theta,phi)');
  title({'f(r,\theta,\phi) - (r- \theta) Plane'});
  view([0,90]);
% print -dpng -r300 func_f1.png % Save image file