% File: pde_1_main.m
% From Chapter 3 - 'Green's Function Analysis', of the book:
% William E Schiesser and Graham W Griffiths (2009).
% A Compendium of Partial Differential Equation Models
% - Method of Lines Analysis with Matlab,
% Cambridge University Press (ISBN-13: 9780521519861).
%
% Clear previous files
  clear all
  clc
%
% Parameters shared with the ODE routine
  global ncall ndss n xl xu x dx
%
% Spatial grid
  n=101;
  xl=-10.0;
  xu= 10.0;
  dx=(xu-xl)/(n-1);
  for i=1:n
    x(i)=xl+dx*(i-1);
  end
  D=1.0;
%
% Initial condition
  n2=(n-1)/2+1;
  for i=1:n
    if(i==n2)u0(i)=25.0*dx;
    else u0(i)=0.0;
    end    
  end   
%
% Independent variable for ODE integration
  t0=0.0;
  tf=2.0;
  tout=(t0:0.5:tf); 
  nout=5;
  ncall=0;
%
% ODE integration
  reltol=1.0e-04; abstol=1.0e-04;
  options=odeset('RelTol',reltol,'AbsTol',abstol);
  mf=3;
  if(mf==1) % explicit FDs 
    [t,u]=ode15s(@pde_1,tout,u0,options); end 
  if(mf==2) ndss=4; % ndss = 2, 4, 6, 8 or 10 required
    [t,u]=ode15s(@pde_2,tout,u0,options); end
  if(mf==3) ndss=44; % ndss = 42, 44, 46, 48 or 50 required 
    [t,u]=ode15s(@pde_3,tout,u0,options); end
%       
% Store analytical solution and the difference between the numerical
% and analytical solutions
  for it=2:nout
  for i=1:n
    u_anal(it,i)=1.0/(2.0*sqrt(D*pi*t(it)))*...
                 exp(-x(i)^2/(4.0*D*t(it)));
    err(it,i)=u(it,i)-u_anal(it,i);
  end
  end
%
% Display selected output
  fprintf('\n mf = %2d   abstol = %8.1e   reltol = %8.1e\n',...
          mf,abstol,reltol);
%
% t = 0 not included
  for it=2:nout
  fprintf('\n     t     x         u(num)        u(anal)            err\n');      
  for i=1:n
    fprintf('%6.2f%6.1f%15.6f%15.6f%15.6f\n',...
            t(it),x(i),u(it,i),u_anal(it,i),err(it,i)); 
  end
%
%   Calculate and display the integral of the solution
    ui=u(it,:);  
    uint=simp(xl,xu,n,ui);  
    fprintf('\n Integral of u(x,t=%4.2f) = %7.4f\n',t(it),uint);  
  end
  fprintf('\n ncall = %4d\n',ncall);  
%
% Plot numerical solution
  figure(1);
  plot(x,u,'-');
  xlabel('x')
  ylabel('u(x,t)')
  title('Green`s function; t = 0, 0.5, 1, 1.5, 2; numerical')
  hold on
%
% Plot numerical and analytical solutions
  figure(2);
  plot(x,u(2:nout,:),'o',x,u_anal,'-');
  xlabel('x')
  ylabel('u(x,t)')
  title('Green`s function; t = 0.5, 1, 1.5, 2; o - numerical; solid - analytical')
 
% Plot numerical solution in 3D perspective
  if(mf==3)
  figure(3);
  view([220 10]);
  hold on
  h1=surfl(x,t,u);
  shading interp;
  colormap gray; 
  hold off
  axis('tight');
  grid on
  zlim([0 0.5])
  ylabel('t, time')
  xlabel('x, distance')
  zlabel('u')
  s1=sprintf('Green`s Function - MOL Solution');
  title(s1, 'fontsize', 12);
  rotate3d on;
  end
% print -deps -r300 pde1.eps; print -dps -r300 pde1.ps; print -dpng -r300 pde1.png