% File: pde_1_main3D.m
% From Chapter 5 - 'Euler, Navier Stokes and Burgers Equations', 
%                   of the book:
% William E Schiesser and Graham W Griffiths (2009).
% A Compendium of Partial Differential Equation Models
% - Method of Lines Analysis with Matlab,
% Cambridge University Press (ISBN-13: 9780521519861).
%%
% The one-dimensional Burgers' equation is
%
%   ut = mu*uxx - u*ux
%
% where mu is a positive constant.
%
% An analytical solution to the PDE is
%
%             0.1*exp(-a) + 0.5*exp(-b) + exp(-c)
%   ua(x,t) = -----------------------------------
%                 exp(-a) + exp(-b) + exp(-c)
%
% where
%   
%   a = (0.05/mu)*(x - 0.5 + 4.95*t)
%
%   b = (0.25/mu)*(x - 0.5 + 0.75*t)
%
%   c = (0.5/mu)*(x - 0.375)
%
% This analytical solution is used to define the initial condition
% and boundary conditions for the numerical solution, i.e.,
%
%   u(x,t=0) = ua(x,t=0)
%
%   u(x=0,t) = ua(x=0,t)
%
%   u(x=1,t) = ua(x=1,t)
%
% Burgers' equation is a comprehensive test problem for the following
% reasons:
%
% (1)  It is a nonlinear PDE (due to the term u*ux)
%
% (2)  The solution, as it evolves through t, undergoes front-sharpening
%      and eventually becomes a finite discontinuity (so that it is a
%      stringent test of any numerical method)
%
% (3)  It has an analytical solution, indicated above, that can be used
%      to evaluate the numerical solution.
%
% Clear previous files
  clear all
  clc
%
% Parameters shared with the ODE routine
  global n x xl xu ncall ncase ndss vis
%
% Select case
%
% ncase = 1 - analytical solution used in BCs
%
% ncase = 2 - homogeneous Neumann BCs
  ncase=2;
%
% Parameters
  n=201;
  xl=0.0;
  xu=1.0;
  vis=0.003;  
%
% Initial condition 
  t0=0.0;
  u0=inital_1(t0);
%
% Independent variable for ODE integration
  tf=1.0;
  tout=[t0:0.01:tf]';
  nout=101;
  ncall=0;
%
% ODE integration 
  reltol=1.0e-04; abstol=1.0e-04;
  options=odeset('RelTol',reltol,'AbsTol',abstol);
  if(ncase==1)
    ndss=4; % ndss = 2, 4, 6, 8 or 10 required
    [t,u]=ode15s(@pde_1,tout,u0,options); end
  if(ncase==2)
    ndss=4; % ndss = 2, 4, 6, 8 or 10 required
    [t,u]=ode15s(@pde_2,tout,u0,options); end
%       
% Store analytical solution, errors
  for it=1:nout
    for i=1:n
      u_anal(it,i)=phi(x(i),t(it));       
      err(it,i)=u(it,i)-u_anal(it,i);
    end     
  end
%
% Display selected output
%   fprintf('\n vis = %8.4f   ndss = %2d\n\n',vis,ndss);
%   for it=1:2:nout
%     fprintf('   it    i   t(it)    x(i)        u(it,i)   u_anal(it,i)      err(it,i)\n');
%     for i=1:5:n  
%       fprintf('%5d%5d%8.2f%8.3f%15.6f%15.6f%15.6f\n',...
%             it,i,t(it),x(i),u(it,i),u_anal(it,i),err(it,i));
%     end
%     fprintf('\n');
%   end
  fprintf('  ncall = %4d\n\n',ncall);
%
% Plot numerical and analytical solutions
  figure(1);
  plot(x,u,'o',x,u_anal,'-')
  xlabel('x')
  ylabel('u(x,t)')
  title('Burgers equation; t = 0, 0.1, ..., 1; o - numerical; solid - analytical')
% print -deps -r300 pde.eps; print -dps -r300 pde.ps; print -dpng -r300 pde.png
%
  figure(2);
  surf(x,t,u,'EdgeColor','none');
  set(get(gca,'XLabel'),'String','space, x')
  set(get(gca,'YLabel'),'String','time, t')
  set(get(gca,'ZLabel'),'String','dependent variable, u(t,x)')
  view(15,15);
  colormap('Cool')
