% File: pde_1_main.m
% From Chapter 6 - 'The Cubic Schrodinger Equation' of the book:
% William E Schiesser and Graham W Griffiths (2009).
% A Compendium of Partial Differential Equation Models
% - Method of Lines Analysis with Matlab,
% Cambridge University Press (ISBN-13: 9780521519861).
%
% The cubic Schrodinger equation (CSE) is 
%
%   i*u + u + q*(abs(u)**2)*u = 0 
%      t   xx
%
% where i = (-1)^0.5.
%
% If a complex function is defined as
%
%   u = v + i*w 
%
% substitution in the CSE gives
%
%   i*(v + i*w ) + (v + i*w ) + q*(v**2 + w**2)*(v + i*w) = 0
%       t     t      xx    xx
%
% or, separating into real and imaginary parts,
%
%   v + w + q*(v**2 + w**2)*w = 0 
%    t   xx
%
%   w - v - q*(v**2 + w**2)*v = 0 
%    t   xx
%
% The CSE is to be integrated subject to the initial condition
%
%   u(x,0) = (2^0.5)*exp(0.5*i*x)*sech(x)
%
% or in terms of v and w
%
%   v(x,0) = (2^0.5)*cos(0.5*x)*sech(x) 
%
%   w(x,0) = (2^0.5)*sin(0.5*x)*sech(x) 
%
% The boundary conditions are taken as
%
%   u(-oo,t) = u(oo,t) = 0 
%
% or
%
%   v(-oo,t) = w(oo,t) = 0 
%
%   v(-oo,t) = w(oo,t) = 0 
%
% In the following coding, we take
%
%   x = -oo as x = -10
%
%   x =  oo as x = 40
%
% The analytical solution to this problem is
%
%   u(x,t) = (2^0.5)*exp(i*(0.5*x + 0.75*t))*sech(x - t) 
%
% and from this analytical solution, we have
%
%   |u(x,t)| = (2^0.5)*sech(x - t) 
%
% which is the equation for a soliton initially centered at
% x = 0, traveling left to right with unit velocity and a
% maximum aplitude of 2^0.5.
%
% Clear previous files
  clear all
  clc %
%  
% Parameters shared with the ODE routine
  global  ncall     x     xl     xu      n
%
% Boundaries, number of grid points
  xl=-10.0;
  xu= 40.0;
  n=251;
%
% Initial condition 
  t0=0.0;
  u0=inital_1(t0);
%
% Independent variable for ODE integration
  tf=30.0;
  tout=[t0:5.0:tf]';
  nout=7;
  ncall=0;
%
% ODE integration
  mf=1;
  reltol=1.0e-06; abstol=1.0e-06;
  options=odeset('RelTol',reltol,'AbsTol',abstol);
%
% Explicit (nonstiff) integration
  if(mf==1)[t,u]=ode45(@pde_1,tout,u0,options); end
%
% Implicit (sparse stiff) integration
  if(mf==2)
    S=jpattern_num;
    pause
    options=odeset(options,'JPattern',S)
    [t,u]=ode15s(@pde_1,tout,u0,options);   
  end    
%    
% One vector to two vectors
  for it=1:nout
  for i=1:n
    v(it,i)=u(it,i);
    w(it,i)=u(it,i+n);
  end  
  end  
%
% Analytical solution and difference between the numerical and
% analytical solutions
  for it=1:nout
    fprintf('     t       x   x - t        v^2+w^2        v^2+w^2          err\n');
    fprintf('                                num            anal              \n')
    for i=1:n
      v2w2_anal(it,i)=ua(t(it),x(i));
      v2w2(it,i)=u(it,i)^2+u(it,i+n)^2;  
      err(it,i)=v2w2(it,i)-v2w2_anal(it,i);
%
%     Output in the neighborhood of the soliton peak
      if(abs(x(i)-t(it))<=1.0)  
        fprintf('%6.2f%8.1f%8.2f%15.6f%15.6f%15.6f\n',...
              t(it),x(i),x(i)-t(it),v2w2(it,i),v2w2_anal(it,i),err(it,i));
      end    
    end
%
%   Calculate and display three invariants
    ui=u(it,:);  
    uint=simp(xl,xu,n,ui);  
    fprintf('\n Invariants at t = %5.2f',t(it));
    fprintf('\n    I1 = %10.4f',    uint(1));
    fprintf('\n    I2 = %10.4f',    uint(2));
    fprintf('\n    I3 = %10.4f\n\n',uint(3));
    fprintf('\n');
  end
  fprintf('  ncall = %4d\n\n',ncall);
%
% Plot numerical and analytical solutions
  figure(1)
  plot(x,v2w2,'o',x,v2w2_anal,'-')
  xlabel('x')
  ylabel('u(x,t)')
  title('CSE; t = 0, 5,..., 30; o - numerical; solid - analytical')
  print -deps -r300 pde.eps; print -dps -r300 pde.ps; print -dpng -r300 pde.png

