% File: pde_1_main.m
% From Chapter 7 - 'The Korteweg-de Vries Equation' of the book:
% William E Schiesser and Graham W Griffiths (2009).
% A Compendium of Partial Differential Equation Models
% - Method of Lines Analysis with Matlab,
% Cambridge University Press (ISBN-13: 9780521519861).
%
% The one-dimensional Korteweg deVries (KdV) equation is
%
%   ut = -uxxx - 6*u*ux
%
% An analytical solution to this PDE is
%
%   ua(x,t) = (1/2)*c*sech^2[(1/2)*c^0.5*(x - c*t)]
%
% where c is the velociy of a soliton produced by this analytical 
% solution.
%   
% This analytical solution is used to define the initial condition
% for the numerical solution, i.e.,
%
%   u(x,t=0) = ua(x,t=0)
%
% The BCs are effectively
%
%   ux(x=-oo,t) = 0
%
%   ux(x=oo,t) = 0
%
% i.e., the spatial domain is taken as large enough that it in
% effect extends over -oo le x le oo.
%
% The KdV equation is a comprehensive test problem for the following
% reasons:
%
% (1)  It is a nonlinear PDE (due to the term -6*u*ux)
%
% (2)  The solution is a soliton that travels without a change in shape
%      so that any dissipation from the numerical method is clear (the
%      numerical "soliton" changes shape)
%
% (3)  It has an analytical solution, indicated above, that can be used
%      to evaluate the numerical solution.
%
% Clear previous files
  clear all
  clc
%
% Parameters shared with other routines
  global  ncall ncase     c    c1    c2    xl    xu     x     n
%
% Select case; 1 - one soliton; 2 - two solitons;
  ncase=2;
  if(ncase==1)c=1.0; n=201; end
  if(ncase==2)c1=2.0; c2=0.5; n=301; end
%
% Initial condition 
  t0=0.0;
  u0=inital_1(t0);
%
% Independent variable for ODE integration
  tf=30.0;
  tout=[t0:10.0:tf]';
  nout=4;
  ncall=0;
%
% ODE integration
  mf=2;
  reltol=1.0e-06; abstol=1.0e-06;
  options=odeset('RelTol',reltol,'AbsTol',abstol);
%
% Explicit (nonstiff) integration
  if(mf==1)[t,u]=ode45(@pde_1,tout,u0,options); end
%
% Implicit (sparse stiff) integration
  if(mf==2)
    S=jpattern_num;
    pause
    options=odeset(options,'JPattern',S)
    [t,u]=ode15s(@pde_1,tout,u0,options);   
  end 
%       
% Store analytical solution, errors in numerical solution  
  if(ncase==1)
    for it=1:nout
      for i=1:n
        u_anal(it,i)=ua(x(i),t(it));       
        err(it,i)=u(it,i)-u_anal(it,i);
      end     
    end
%
%   Display selected output
    fprintf('\n ncase = %2d   c = %5.2f\n',ncase,c);
    for it=1:nout
      fprintf('     t       x        u(it,i)   u_anal(it,i)      err(it,i)\n');
      for i=1:5:n  
        fprintf('%6.2f%8.3f%15.6f%15.6f%15.6f\n',...
              t(it),x(i),u(it,i),u_anal(it,i),err(it,i));
      end
%
%     Calculate and display three invariants
      ui=u(it,:);  
      uint=simp(xl,xu,n,ui);  
      fprintf('\n Invariants at t = %5.2f',t(it));
      fprintf('\n    I1 = %10.4f   Mass conservation'    ,uint(1));
      fprintf('\n    I2 = %10.4f   Energy conservation'  ,uint(2));
      fprintf('\n    I3 = %10.4f   Whitham invariant\n\n',uint(3));
    end
    fprintf('    ncall = %4d\n\n',ncall);
%
%   Plot numerical and analytical solutions
    plot(x,u,'o',x,u_anal,'-')
    xlabel('x')
    ylabel('u(x,t)')
    title('KdV equation; t = 0, 10, 20, 30; o - numerical; solid - analytical')
    print -deps -r300 pde2.eps; print -dps -r300 pde2.ps; print -dpng -r300 pde2.png
  end
%       
% Store numerical solution
  if(ncase==2)
%
%   Display selected output
    fprintf('\n ncase = %2d   c1 = %5.2f   c2 = %5.2f\n',ncase,c1,c2);
    for it=1:nout
      fprintf('     t       x        u(it,i)\n');
      for i=1:5:n  
        fprintf('%6.2f%8.3f%15.6f\n',t(it),x(i),u(it,i));
      end
      fprintf('\n');
%
%     Calculate and display three invariants
      ui=u(it,:);  
      uint=simp(xl,xu,n,ui);  
      fprintf('\n Invariants at t = %5.2f',t(it));
      fprintf('\n    I1 = %10.4f   Mass conservation'    ,uint(1));
      fprintf('\n    I2 = %10.4f   Energy conservation'  ,uint(2));
      fprintf('\n    I3 = %10.4f   Whitham invariant\n\n',uint(3));
    end    
    fprintf('  ncall = %4d\n\n',ncall);
%
%   Plot numerical solution
    for it=1:nout
      subplot(2,2,it)
      plot(x,u(it,:),'-')
      axis([-30 70 0 1])
      xlabel('x')
      if(it==1)
        ylabel('u(x,0)')
        title('KdV; t = 0')
      elseif(it==2)
        ylabel('u(x,10)')
        title('KdV; t = 10')
      elseif(it==3)
        ylabel('u(x,20)')
        title('KdV; t = 20')
      elseif(it==4)
        ylabel('u(x,30)')
        title('KdV; t = 30')
      end  
      print -deps -r300 pde3.eps; print -dps -r300 pde3.ps; print -dpng -r300 pde3.png
    end
  end  
  

