% File: pde_1_main.m
% From Chapter 11 - 'Tree-Dimensional Partial Differential Equation',
%                    of the book:
% William E Schiesser and Graham W Griffiths (2009).
% A Compendium of Partial Differential Equation Models
% - Method of Lines Analysis with Matlab,
% Cambridge University Press (ISBN-13: 9780521519861).
%
% Laplace's equation (an elliptic PDE)
%
%   0 = uxx + uyy
%
% with boundary conditions
%
%   ux(x=0,y) = ux(x=1,y) = 0
%
%   u(x,y=0) = 0, u(x,y=1) = 1
%
% This elliptic problem is solved numerically via the associated
% parabolic problem
%
%   ut = uxx + uyy
%
% with the same boundary conditions and the "initial condition"
%
%   u(x,y,t=0) = u0
%
% The analytical solution to Laplace's equation (for t ---> oo) is
%
%   u(x,y) = y for all x
%
% i.e., a linear variation in y for all x.  More complicated
% solutions can be produced by changing the BCs, but an analytical
% may not be readily available, even at steady state.
%
% Clear previous files
  clear all
  clc
%
% Parameters shared with the ODE routine
  global ncall nx ny ndss mmf

% Initial condition (which also is consistent with the boundary
% conditions)
  nx=11;
  ny=11;
  for i=1:nx
  for j=1:ny
    if(j==1)     u0(i,j)=0.0;
    elseif(j==ny)u0(i,j)=1.0;    
    else         u0(i,j)=0.5; 
    end    
  end 
  end
%
% Matrix conversion method flag
%
%   mmf = 1 - explicit subscripting for matrix conversion
%
%   mmf = 2 - Matlab reshape function
%
  mmf=2;
%
% 2D to 1D matrix conversion 
%
  if(mmf==1)
    for i=1:nx
    for j=1:ny    
      y0((i-1)*ny+j)=u0(i,j); 
    end
    end
  end
%  
  if(mmf==2)
    y0=reshape(u0',1,nx*ny);
  end
%
% Independent variable for ODE integration
  t0=0.0;
  tf=0.15;
  tout=[t0:0.03:tf]'; 
  nout=6;
  ncall=0;
%
% ODE integration 
  mf=2;
  reltol=1.0e-04; abstol=1.0e-04;
  options=odeset('RelTol',reltol,'AbsTol',abstol);
  if(mf==1) % explicit FDs 
    [t,y]=ode15s(@pde_1,tout,y0,options); end 
  if(mf==2) ndss=4; % ndss = 2, 4, 6, 8 or 10 required
    [t,y]=ode15s(@pde_2,tout,y0,options); end
  if(mf==3) ndss=44; % ndss = 42, 44, 46, 48 or 50 required 
    [t,y]=ode15s(@pde_3,tout,y0,options); end
%
% 1D to 2D matrix conversion (plus t), with 3D plotting at t = 0,
% 0.03, ..., 0.15
%
% Composite 3D plots showing evolution of solution
  figure(1);
  for it=1:nout
%
% 2D to 3D matrix conversion
    u(it,:,:)=reshape(y(it,:),ny,nx)';
    subplot(3,2,it)
    uview(:,:)=u(it,:,:);
    surf(uview);
  end
%
% 3D plot of initial condition
  figure(2);
  uview(:,:)=u(1,:,:);
  surf(uview);
  xlabel('y grid number');
  ylabel('x grid number');
  zlabel('u(x,y)');
  s1=sprintf('Laplace Equation - MOL Solution');
  s2=sprintf('Parametric plot at t=0 - initial condition');
  title([{s1}, {s2}], 'fontsize', 12);
  rotate3d on;    
%
% 3D plot of final solution
  figure(3);
  uview(:,:)=u(nout,:,:);
  surf(uview);
  xlabel('y grid number');
  ylabel('x grid number');
  zlabel('u(x,y)');
  s1=sprintf('Laplace Equation - MOL Solution');
  s2=sprintf('Parametric plot at t=%4.2f - final solution',tout(nout));
  title([{s1}, {s2}], 'fontsize', 12);
  rotate3d on;   
%
% Display selected output
  for it=1:nout
    fprintf('\n\n t = %5.2f',t(it));
    fprintf('\n x across (x=0,0.2,...,1.0)');
    fprintf('\n y  down  (y=1.0,0.8,...,0)');
  for j=ny:-2:1
    fprintf('\n%10.3f%10.3f%10.3f%10.3f%10.3f%10.3f',u(it,1:2:nx,j));
  end
  end
  fprintf('\n\n ncall = %4d\n',ncall);
