%
% Laplace's equation in three dimensions (an elliptic PDE)
%
%   0 = uxx + uyy + uzz
%
% with boundary conditions
%
%   u(x=0,y,z) = u(x=1,y,z) = 1
%
%   uy(x,y=0,z) = uy(x,y=1,z) = 0
%
%   uz(x,y,z=0) + u(x,y,z=0) = 1, uz(x,y,z=1) + u(x,y,z=1) = 1
%
% This elliptic problem is solved numerically via the associated
% parabolic problem
%
%   ut = uxx + uyy + uzz
%
% with the same boundary conditions and the "initial condition"
%
%   u(x,y,z,t=0) = u0(x,y,z)
%
% For the particular BCs indicated above, the steady state solution 
% of the parabolic problem is
%
%   u(x,y,z,t=oo) = 1
%
% More interesting solutions (other than a constant) can be produced
% by changing the BCs, but an analytical solution might not be
% readily available.
%
% Clear previous files
  clear all
  clc
%
% Parameters shared with the ODE routine
  global ncall nx ny nz ndss

% Initial condition (which also is consistent with the boundary
% conditions)
  nx=15;
  ny=15;
  nz=15;
  u0=zeros(nx,ny,nz);
  for i=1:nx
  for j=1:ny
  for k=1:nz    
    if(i==1),     u0(i,j,k)=1.0;
    elseif(i==nx),u0(i,j,k)=1.0;
    else          u0(i,j,k)=0.0; 
    end    
  end 
  end
  end
% 
% 3D to 1D matrix conversion
  %y0=reshape(u0,1,nx*ny*nz);
  for i=1:nx
  for j=1:ny
  for k=1:nz
    y0((i-1)*ny*nz+(j-1)*nz+k)=u0(i,j,k);
  end
  end
  end
%
% Independent variable for ODE integration
  t0=0.0;
  tf=1.0;
  nout = 25;
  tout = linspace(t0,tf,nout);
  % tout=(t0:0.1:tf)'; 
  % nout=length(tout);
  ncall=0;
%
% ODE integration 
  mf=3;
  reltol=1.0e-04; abstol=1.0e-04;
  options=odeset('RelTol',reltol,'AbsTol',abstol);
  if(mf==1) % explicit FDs 
    [t,y]=ode15s(@pde_1,tout,y0,options); end 
  if(mf==2), ndss=4; % ndss = 2, 4, 6, 8 or 10 required
    [t,y]=ode15s(@pde_2,tout,y0,options); end
  if(mf==3), ndss=44; % ndss = 42, 44, 46, 48 or 50 required 
    [t,y]=ode15s(@pde_3,tout,y0,options); end
%
% 1D to 3D matrix conversion
%   for it=1:nout
%     u(it,:,:,:)=reshape(y(it,:),nx,ny,nz);
%   end
  for it=1:nout
  for i=1:nx
  for j=1:ny
  for k=1:nz
    u(it,i,j,k)=y(it,(i-1)*ny*nz+(j-1)*nz+k);
  end
  end
  end
  end

% Display selected output as t, 
%   u(t,x=0,y=0.5,z=0.5)
%   u(t,x=1,y=0.5,z=0.5)
%   u(t,x=0.5,y=0,z=0.5)
%   u(t,x=0.5,y=1,z=0.5)
%   u(t,x=0.5,y=0.5,z=0)
%   u(t,x=0.5,y=0.5,z=1)
%   u(t,x=0.5,y=0.5,z=0.5)
  fprintf('\n mf = %2d\n',mf);
  for it=1:2:nout
    fprintf('\n t = %5.3f\n%10.3f%10.3f\n%10.3f%10.3f\n%10.3f%10.3f\n%10.3f\n',...
            t(it),u(it,1,6,6),u(it,nx,6,6),u(it,6,1,6),u(it,6,ny,6)...
                 ,u(it,6,6,1),u(it,6,6,nz),u(it,6,6,6));
  end
  fprintf('\n ncall = %4d\n',ncall);

  
  
