% File: pde_1_main.m
% From Chapter 12 - 'Partial Differential Equation with a Mixed Partial
%                    Derivative', of the book:
% William E Schiesser and Graham W Griffiths (2009).
% A Compendium of Partial Differential Equation Models
% - Method of Lines Analysis with Matlab,
% Cambridge University Press (ISBN-13: 9780521519861).
% The following PDE with a mixed partial derivative
%
%   u  = -u    + u                                         (1)
%    t     xxt
%
% is integrated numerically by the method of lines (MOL).
%
% The initial condition (IC) for eq. (1) is
%
%   u(x,t=0) = f(x) = sin(x*pi/L)                          (2)
%
% and the boundary conditions (BCs) are
%
%   u(x=0,t) = u(x=L,t) = 0                             (3)(4)
%
% The analytical solution to this problem is
%
%   u(x,t) = sin(x*pi/L*exp(a*t)                           (5)
%
% where
%
%   a = 1/(1 - (pi/L)^2)
%
% The analytical solution, eq. (5), is compared with the
% numerical solution of eqs. (1) to (4) in the subsequent
% programming.
%
% Clear previous files
  clear all
  clc 
%  
% Parameters shared with the ODE routine
  global  a     x    xl    xu    dx    cm     n  ncall
%
% Boundaries, number of grid points
  xl=0.0;
  xu=1.0;
  n=49;
%
% Initial condition 
  t0=0.0;
  u0=inital_1(t0);
%
% Independent variable for ODE integration
  tf=20.0;
  tout=[t0:4.0:tf]';
  nout=6;
  ncall=0;
%
% Coefficient matrix
  for i=1:n
  for j=1:n
    if(i==j)cm(i,j)=(1.0-2.0/dx^2);
    elseif(abs(i-j)==1)cm(i,j)=1.0/dx^2;
    else cm(i,j)=0.0; 
    end    
  end
  end
%
% ODE integration
  reltol=1.0e-06; abstol=1.0e-06;
  options=odeset('RelTol',reltol,'AbsTol',abstol);
%
% Explicit (nonstiff) integration
  [t,u]=ode45(@pde_1,tout,u0,options);
%
% Analytical solution and difference between the numerical and
% analytical solutions at selected points
  for it=1:nout
    fprintf('\n\n     t       x      u(x,t)      u(x,t)         err\n')
    fprintf('                     num         anal             \n')
    for i=1:n
      u_anal(it,i)=ua(t(it),x(i));
      err(it,i)=u(it,i)-u_anal(it,i);
    end    
    for i=1:5:n
      fprintf('%6.2f%8.3f%12.6f%12.6f%12.6f\n',...
              t(it),x(i),u(it,i),u_anal(it,i),err(it,i));
    end          
  end
  fprintf('\n  ncall = %4d\n\n',ncall);
%
% Plot numerical and analytical solutions
  figure(1)
  x=[0.0 x 1.0];
  for it=1:nout
    uplot(it,1)=0.0;
    uplot(it,2:n+1)=u(it,1:n);
    uplot(it,n+2)=0.0;
    uaplot(it,1)=0.0;
    uaplot(it,2:n+1)=u_anal(it,1:n);
    uaplot(it,n+2)=0.0;
  end  
  plot(x,uplot,'o',x,uaplot,'-')
  xlabel('x')
  ylabel('u(x,t)')
  title('Mixed partial PDE; t = 0, 5,..., 20; o - numerical; solid - analytical')
  print -deps -r300 pde.eps; print -dps -r300 pde.ps; print -dpng -r300 pde.png

