% File: pde_1_main.m
% From Chapter 13 - 'Simultaneous, Nonlinear, Two-Dimensional Partial
%                    Differential Equations in Cylindrical Coordinates',
%                    of the book:
% William E Schiesser and Graham W Griffiths (2009).
% A Compendium of Partial Differential Equation Models
% - Method of Lines Analysis with Matlab,
% Cambridge University Press (ISBN-13: 9780521519861).
%
% Dynamic analysis of a plug flow reactor
%
% The following equations model a plug flow reactor (PFR)
%
%   Material balance
%
%   ca  = -v*ca  + D*(Ca   + (1/r)Ca ) - k*ca^2                 (1)
%     t        z        rr          r
%
%   Energy balance
%
%   T  = -v*T  + k*/(rho*Cp)*(T   + (1/r)*T )                   (2)
%    t       z                 rr          r
%
%        - (dH*k/(rho*Cp)*ca^2
%
%   kr = k0*exp(-E/(R*T))                                       (3)
%
% The variables and parameters for this model are (in cgs units)
%
%   Reactant concentration          ca        (eq. (1))
%
%   Temperature                     T         (eq. (2))
%
%   Reaction rate constant          kr        (eq. (3))
%
%   Time                            t
%
%   Radial position                 r
%
%   Axial position                  z
%
%   Entering concentration          ca0          0.01
%
%   Entering temperature            T0           305
%
%   Wall temperature                TW           355
%                                             
%   Reactor radius                  r0           2
%
%   Reactor length                  zl           100
%
%   Linear velocity                 v            1
%
%   Mass diffusivity                Dc           0.1
%
%   Thermal diffusivity       Dt  = k/(rho*Cp)   0.1
%
%   Liquid density                  rho          1.0
%
%   Liquid specific heat            Cp           0.5
%
%   Heat of reaction                dH          -10,000
%                                                       
%   Specific rate constant          k0           1.5e+09
%                                               
%   Activation energy               E            15,000
%
%   Gas constant                    R            1.987
%
% Note that there are two cases, corresponding to the two
% values of the specific rate constant, k0.  This is a 
% sensitive parameter since it multiplies a stongly nonlinear
% term (Arrhenius temperature dependency) in eq. (3).  Thus,
% two values are programmed k0 = 1.5e+09, 2.0e+09, to indicate
% the effect of k0.
%
% The solution to this system, ca(r,z,t) (from eq. (1)) and
% T(r,z,t) (from eq. (2)) is computed by a fixed step Euler
% integration in t.  Also, the choice of an Euler step is
% sensitive because of the nonlinearity of eq. (3).
%
% Clear previous files
  clear all
  clc
%
% Global area
  global     nr     nz     dr     dz    drs    dzs...
              r     r0      z     zl     Dc     Dt...
             ca     Tk    cae    Tke      h      k...
              E      R    rk0      v    rho     Cp...
             Tw     dH  ncall  
%
% Model parameters
  ca0=0.0;
  cae=0.01;
  Tk0=305.0;
  Tke=305.0;
  Tw=355.0; 
  r0=2.0;
  zl=100.0;
  v=1.0;
  Dc=0.1;
  Dt=0.1;
  k=0.01;
  h=0.01;
  rho=1.0;
  Cp=0.5;
  rk0= 1.5e+09;
  dH=-10000.0;
  E=15000.0;
  R=1.987;   
%
% Grid in axial direction
  nz=20;
  dz=zl/nz;
  for i=1:nz
    z(i)=i*dz;
  end
%
% Grid in radial direction
  nr=7;
  dr=r0/(nr-1);
  for j=1:nr
    r(j)=(j-1)*dr;
  end
  drs=dr^2;  
%
% Independent variable for ODE integration
  tf=200.0;
  tout=[0.0:50.0:tf]'; 
  nout=5;
  ncall=0;
%
% Initial condition
  for i=1:nz
  for j=1:nr
    ca(i,j)=ca0;
    Tk(i,j)=Tk0;
    y0((i-1)*nr+j)=ca(i,j);
    y0((i-1)*nr+j+nz*nr)=Tk(i,j);
  end
  end
%
% ODE integration 
  reltol=1.0e-04; abstol=1.0e-04;
  options=odeset('RelTol',reltol,'AbsTol',abstol);
  mf=2;
  if(mf==1)[t,y]=ode15s(@pde_1,tout,y0,options);end 
  if(mf==2)[t,y]=ode15s(@pde_2,tout,y0,options);end 
%
%   1D to 2D matrices
  for it=1:nout
  for i=1:nz
  for j=1:nr  
    ca(it,i,j)=y(it,(i-1)*nr+j);
    Tk(it,i,j)=y(it,(i-1)*nr+j+nz*nr);
  end
  end
  end
%
% Display a heading and centerline output
  fprintf('\n  nr = %2d   nz = %2d\n',nr,nz);
  for it=1:nout
    fprintf('\n t = %4.1f\n',t(it));  
  for i=1:nz
    fprintf(' z = %5.1f   ca(r=0,z,t) = %8.5f   Tk(r=0,z,t) = %8.2f\n',...
        z(i),ca(it,i,1),Tk(it,i,1));
  end
  end
  fprintf('\n ncall = %5d\n',ncall);
%
% Parametric plots
%
% Axial profiles
  figure(1);
  subplot(2,2,1)
  plot(z,ca(2,:,1)); axis([0 zl 0 0.01]);
  title('ca(r=0,z,t=50)'); xlabel('z'); ylabel('ca(r=0,z,t=50)')
  subplot(2,2,2)
  plot(z,Tk(2,:,1)); axis([0 zl 305 450]);
  title('Tk(r=0,z,t=50)'); xlabel('z'); ylabel('Tk(r=0,z,t=50)')  
  subplot(2,2,3)
  plot(z,ca(3,:,1)); axis([0 zl 0 0.01]);
  title('ca(r=0,z,t=100)'); xlabel('z'); ylabel('ca(r=0,z,t=100)')
  subplot(2,2,4)
  plot(z,Tk(3,:,1)); axis([0 zl 305 450]);
  title('Tk(r=0,z,t=100)'); xlabel('z'); ylabel('Tk(r=0,z,t=100)')
  figure(2);
  subplot(2,2,1)
  plot(z,ca(4,:,1)); axis([0 zl 0 0.01]);
  title('ca(r=0,z,t=150)'); xlabel('z'); ylabel('ca(r=0,z,t=150)')
  subplot(2,2,2)
  plot(z,Tk(4,:,1)); axis([0 zl 305 450]);
  title('Tk(r=0,z,t=150)'); xlabel('z'); ylabel('Tk(r=0,z,t=150)')    
  subplot(2,2,3)
  plot(z,ca(5,:,1)); axis([0 zl 0 0.01]);
  title('ca(r=0,z,t=200)'); xlabel('z'); ylabel('ca(r=0,z,t=200)')
  subplot(2,2,4)
  plot(z,Tk(5,:,1)); axis([0 zl 305 450]);
  title('Tk(r=0,z,t=200)'); xlabel('z'); ylabel('Tk(r=0,z,t=200)')  
%
% Radial profiles (at t = tf)
  for i=1:nz,
  for j=1:nr
    ca_rad(i,j)=ca(5,i,j);
    Tk_rad(i,j)=Tk(5,i,j);
  end
  end
  figure(3);
  subplot(2,2,1)
  plot(r,ca_rad(1,:)); axis([0 r0 0.0 0.01]);
  title('ca(r,z=5,t=200)'); xlabel('r'); ylabel('ca(r,z=5,t=200)')
  subplot(2,2,2)
  plot(r,ca_rad(7,:)); axis([0 r0 0.0 0.01]);
  title('ca(r,z=35,t=200)'); xlabel('r'); ylabel('ca(r,z=35,t=200)')    
  subplot(2,2,3)
  plot(r,ca_rad(14,:)); axis([0 r0 0.0 0.01]);
  title('ca(r,z=70,t=200)'); xlabel('r'); ylabel('ca(r,z=70,t=200)')
  subplot(2,2,4)
  plot(r,ca_rad(20,:)); axis([0 r0 0.0 0.01]);
  title('ca(r,z=100,t=200)'); xlabel('r'); ylabel('ca(r,z=100,t=200)') 
  figure(4);
  subplot(2,2,1)
  plot(r,Tk_rad(1,:)); axis([0 r0 305 450]);
  title('Tk(r,z=5,t=200)'); xlabel('r'); ylabel('Tk(r,z=5,t=200)')
  subplot(2,2,2)
  plot(r,Tk_rad(7,:)); axis([0 r0 305 450]);
  title('Tk(r,z=35,t=200)'); xlabel('r'); ylabel('Tk(r,z=35,t=200)')    
  subplot(2,2,3)
  plot(r,Tk_rad(14,:)); axis([0 r0 305 450]);
  title('Tk(r,z=70,t=200)'); xlabel('r'); ylabel('Tk(r,z=70,t=200)')
  subplot(2,2,4)
  plot(r,Tk_rad(20,:)); axis([0 r0 305 450]);%
%
% 3D plotting
  title('Tk(r,z=100,t=200)'); xlabel('r'); ylabel('Tk(r,z=100,t=200)') 
  figure()
  surf(r,z,Tk_rad)
  axis([0 r0 0 zl 0 500]);
  xlabel('r - radial distance');
  ylabel('z - axial distance');
  zlabel('Tk - reactant temperature');
  title([{'Surface & contour plot of reactant temperature, Tk(r,z),'}, ...
                        { 'at time t=200.0'}]);
  view(-130,62);
  figure()
  surf(r,z,ca_rad)
  axis([0 r0 0 zl 0 0.01]);
  xlabel('r - radial distance');
  ylabel('z - axial distance');
  zlabel('ca - reactant concentration');
  title([{'Surface & contour plot of reactant concentration, ca(r,z),'}, ...
                        { 'at time t=200.0'}]);
  view(-130,62);
  rotate3d on

  
