% File: pde_1_main_3D.m
% From Chapter 14 - 'Diffusion Equation in Spherical Coordinates',
%                    of the book:
% William E Schiesser and Graham W Griffiths (2009).
% A Compendium of Partial Differential Equation Models
% - Method of Lines Analysis with Matlab,
% Cambridge University Press (ISBN-13: 9780521519861).
%
% Produces the 3D plots
%
% Clear previous files
  clear all
  clc
%
% Global area
  global r r0 nr D f tau ncall
%
% Model parameters
  D=0.1;
  r0=1.0;
  std=1.0;
  tau=1.0;
%
% Radial grid and inhomogeneous term
  nr=21;
  dr=r0/(nr-1);
  for i=1:nr
    r(i)=(i-1)*dr;
    f(i)=exp(-r(i)^2/std^2);
  end
%
% Independent variable for ODE integration
  tf=2.5;
  tout=[0.0:0.25:tf]'; 
  nout=length(tout);
  ncall=0;
%
% Initial condition
  u0=zeros(nr,1);
%
% ODE integration 
  reltol=1.0e-04; abstol=1.0e-04;
  options=odeset('RelTol',reltol,'AbsTol',abstol);
  [t,u]=ode15s(@pde_1,tout,u0,options);
%
% Display a heading and selected output
  fprintf('\n nr = %2d   r0 = %4.2f   std = %4.2f\n',nr,r0,std);
  for it=1:nout
    fprintf('\n t = %4.2f\n',t(it));  
    for i=1:2:nr
      fprintf(' r = %4.1f   u(r,t) = %8.5f\n',r(i),u(it,i));
    end
    for i=1:nr
      u1d(i)=4.0*pi*r(i)^2*u(it,i);
    end
    I1=simp(0.0,r0,nr,u1d)/(4.0/3.0*pi*r0^3);
    fprintf(' \n integral = %7.5f\n',I1);
  end
  fprintf('\n ncall = %5d\n',ncall);
%
% Parametric plot
  figure(1);
  plot(r,u); axis([0 r0 0 1]);
  title('u(r,t), t = 0, 0.25, ..., 2.5'); xlabel('r'); ylabel('u(r,t)')
%
% Extend integration for closer approach to steady state
  fprintf('\n Extended solution\n');
  u0=u(nout,:);
  t0=t(nout);
  tf=t0+5.0;
  tout=[t0:0.25:tf]'; 
  nout=length(tout);        
  [t1,u1]=ode15s(@pde_1,tout,u0,options);
  for it=1:nout
    fprintf('\n t = %4.1f\n',t1(it));  
    for i=1:2:nr
      fprintf(' r = %4.1f   u(r,t) = %8.5f\n',r(i),u1(it,i));
    end
    for i=1:nr
      u1d(i)=4.0*pi*r(i)^2*u1(it,i);
    end
    I1=simp(0.0,r0,nr,u1d)/(4.0/3.0*pi*r0^3);
    fprintf(' \n integral = %7.5f\n',I1);
  end
  fprintf('\n ncall = %5d\n',ncall);
  
  tt=[t;t1];
  u2=[u;u1];
  surf(r, tt ,u2);
  axis([0 r0 0 tf 0 0.8]);
  xlabel('radial distance');
  ylabel('time');
  zlabel('u - dependent variable');
  title([{'Surface plot of independent variable, u(r,\theta),'}, ...
                        { 'at time t=7.5'}]);
  rotate3d on
  
  
  
