% File: pde_2_main_3D.m
% From Chapter 14 - 'Diffusion Equation in Spherical Coordinates',
%                    of the book:
% William E Schiesser and Graham W Griffiths (2009).
% A Compendium of Partial Differential Equation Models
% - Method of Lines Analysis with Matlab,
% Cambridge University Press (ISBN-13: 9780521519861).
%
% Produces 3D plots
%
% Clear previous files
  clear all
  clc
%
% Global area
  global r r0 nr th th0 nth D std_0 std_pi2 f tau ncall
%
% Model parameters
  D=0.1;
  r0=2.0;
  th0=pi; %/2.0;
  std_0  =2.0;
  std_pi2=1.0;
  tau=1.0;
%
% Independent variable for ODE integration
  tf=1.0;
  tout=[0.0:0.25:tf]'; 
  nout=length(tout);
  ncall=0;
%
  nr=21;
  nth=21;
% Preallocate arrays
  y0=zeros(1,nr*nth);
  f=zeros(nr,nth);
  r=zeros(1,nr);
  th=zeros(1,nth);
  u=zeros(nout,nr,nth);
  %
% Radial grid
  dr=r0/(nr-1);
  for i=1:nr
    r(i)=(i-1)*dr;
  end
%
% Angular grid
  dth=th0/(nth-1);
  for j=1:nth
    th(j)=(j-1)*dth;
  end  
%
% Inhomogeneous term
  for i=1:nr
  for j=1:nth
    f(i,j)=exp(-(r(i)*sin(th(j))/std_pi2)^2 ...
               -(r(i)*cos(th(j))/std_0  )^2);
  end
  end
%
% Initial condition
  for i=1:nr
  for j=1:nth
    y0((i-1)*nth+j)=0.0;
  end
  end
%
% ODE integration 
  reltol=1.0e-04; abstol=1.0e-04;
  options=odeset('RelTol',reltol,'AbsTol',abstol);
  [t,y]=ode15s(@pde_2,tout,y0,options); 
%
% 1D to 2D (3D with t)
  for it=1:nout
  for  i=1:nr
  for  j=1:nth
     u(it,i,j)=y(it,(i-1)*nth+j);
  end
  end
  end
%
% Display a heading and selected output
  fprintf('\n  nr = %2d    r0 = %4.2f', nr, r0);
  fprintf('\n nth = %2d   th0 = %4.2f',nth,th0);
  fprintf('\n   std_0 = %4.2f  ',std_0);
  fprintf('\n std_pi2 = %4.2f\n',std_pi2);  
  for it=1:nout
    fprintf('\n t = %4.2f',t(it));  
    for i=1:2:nr
      fprintf('\n');  
    for j=1:2:nth    
      fprintf(' r = %4.1f   th = %4.2f   u(r,th,t) = %8.5f\n',...
              r(i),th(j),u(it,i,j));
    end
    end
  end
  fprintf('\n ncall = %5d\n',ncall);
%
% Parametric plots
  figure(1);
  subplot(2,2,1)
  uplot(1:nr,1:5:nth)=u(2,1:nr,1:5:nth);
  plot(r,uplot); axis([0 r0 0 1]);
  title('u(r,th,t), th = 0, \pi/4, \pi/2; t = 0.25'); 
  xlabel('r'); ylabel('u(r,th,t)');
  subplot(2,2,2)
  uplot(1:nr,1:5:nth)=u(3,1:nr,1:5:nth);
  plot(r,uplot); axis([0 r0 0 1]);
  title('u(r,th,t), th = 0, \pi/4, \pi/2; t = 0.5'); 
  xlabel('r'); ylabel('u(r,th,t)');
  subplot(2,2,3)
  uplot(1:nr,1:5:nth)=u(4,1:nr,1:5:nth);
  plot(r,uplot); axis([0 r0 0 1]);
  title('u(r,th,t), th = 0, \pi/4, \pi/2; t = 0.75'); 
  xlabel('r'); ylabel('u(r,th,t)');
  subplot(2,2,4)
  uplot(1:nr,1:5:nth)=u(5,1:nr,1:5:nth);
  plot(r,uplot); axis([0 r0 0 1]);
  title('u(r,th,t), th = 0, \pi/4, \pi/2; t = 1'); 
  xlabel('r'); ylabel('u(r,th,t)');
  
% 3D plots
  % Create mesh for two-quadrant images
  [R1, Theta1]=meshgrid(r, th);
  u1plt=zeros(nr,nth);
  figure(2)
  subplot(2,2,1)
  u1plt(:,:)=u(2,1:nr,1:nth);
  [X1,Y1,U0]=pol2cart(Theta1,R1,u1plt);
  surf(Y1,X1,U0')
  caxis([0 1])
  axis([0, r0, -r0, r0, 0, 1]); axis equal
  view(-12,10);
  rotate3d on
  title('u(r,th,t), t = 0.25'); 
  xlabel('r'); ylabel('r');zlabel('u');
  %
  subplot(2,2,2)
  u1plt(:,:)=u(3,1:nr,1:nth);
  [X1,Y1,U0]=pol2cart(Theta1,R1,u1plt);
  surf(Y1,X1,U0');
  caxis([0 1])
  axis([0, r0, -r0, r0, 0, 1]); axis equal
  view(-12,10);
  rotate3d on
  title('u(r,th,t), t = 0.5'); 
  xlabel('r'); ylabel('r');zlabel('u');
  %
  subplot(2,2,3)
  u1plt(:,:)=u(4,1:nr,1:nth);
  [X1,Y1,U0]=pol2cart(Theta1,R1,u1plt);
  surf(Y1,X1,U0');
  caxis([0 1])
  axis([0, r0, -r0, r0, 0, 1]);axis equal
  view(-12,10);
  rotate3d on
  title('u(r,th,t), t = 0.75'); 
  xlabel('r'); ylabel('r');zlabel('u');
  %
  subplot(2,2,4)
  u1plt(:,:)=u(5,1:nr,1:nth);
  [X1,Y1,U0]=pol2cart(Theta1,R1,u1plt);
  surf(Y1,X1,U0');
  caxis([0 1])
  axis([0, r0, -r0, r0, 0, 1]);axis equal
  view(-12,10);
  rotate3d on
  title('u(r,th,t), t = 1.0'); 
  xlabel('r'); ylabel('r');zlabel('u');
  %
  
  


  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
